{% templatetag openblock %} load horizon {% templatetag closeblock %}
{% templatetag openblock %} jstemplate {% templatetag closeblock %}
{% templatetag openblock %} extends 'base.html' {% templatetag closeblock %}

{% templatetag openblock %} block sidebar {% templatetag closeblock %}
  {% templatetag openblock %} include 'horizon/common/_sidebar.html' {% templatetag closeblock %}
{% templatetag openblock %} endblock {% templatetag closeblock %}

{% templatetag openblock %} block main {% templatetag closeblock %}
  {% templatetag openblock %} include "horizon/_messages.html" {% templatetag closeblock %}
  {% templatetag openblock %} block {{ dash_name }}_main {% templatetag closeblock %}{% templatetag openblock %} endblock {% templatetag closeblock %}
{% templatetag openblock %} endblock {% templatetag closeblock %}
{% templatetag openblock %} endjstemplate {% templatetag closeblock %}
