themepreview {

  #source-button {
    font-weight: 700;
    position: absolute;
    right: 0;
    top: 0;
    z-index: 100;
  }

  .bs-component {
    position: relative;
  }

  .bs-component .modal {
    bottom: auto;
    display: block;
    left: auto;
    position: relative;
    right: auto;
    top: auto;
    z-index: 1;
  }

  .bs-component .popover {
    display: inline-block;
    margin: 20px;
    position: relative;
    width: 220px;
  }

  .left .tooltip-inner {
    float: right;
  }

  #nav-tabs .nav-tabs {
    margin-bottom: 15px;
  }

  .nav-pills.nav-pills-stacked {
    max-width: 300px;
  }

  .navbar {
    margin-bottom: $padding-large-horizontal*2;
  }

  .bs-container {
    margin-bottom: 15px;

    .bs-component {
      margin: 0;
    }
  }

  .page-header {

    h1:before {
      content: '';
      height: $navbar-height;
      margin: -$navbar-height 0 0;
      display: block;
    }
  }

  .line-chart {
    height: 100% !important; // :( to override inline height: 0
  }
}
