{% extends "horizon/common/_modal_form.html" %}
{% load i18n %}

{% block modal-body-right %}
  <h3>{% trans "Description:" %}</h3>
  <p>{% blocktrans trimmed %}
    Security groups are sets of IP filter rules that are applied to
    network interfaces of a VM. After the security group is created,
    you can add rules to the security group.
    {% endblocktrans %}
  </p>
{% endblock %}
