{% load branding i18n %}
{% load context_selection %}
{% load themes %}

<nav class="navbar-inverse material-header navbar-fixed-top">
  <div class="container-fluid">
    <!-- Brand and toggle get grouped for better mobile display -->
    <div class="navbar-header">
      <button type="button" class="navbar-toggle collapsed" data-toggle="collapse" data-target="#navbar-collapse">
        <span class="sr-only">{% trans "Toggle navigation" %}</span>
        <span class="icon-bar"></span>
        <span class="icon-bar"></span>
        <span class="icon-bar"></span>
      </button>
      <div class="md-hamburger">
        <button class="md-hamburger-trigger">
          <span class="md-hamburger-layer md-hamburger-menu"></span>
        </button>
        {% theme_dir as theme_dir %}
        <script src='{{ STATIC_URL }}{{ theme_dir }}/material/js/material.hamburger.js'></script>
      </div>
      {% include "header/_brand.html" %}
    </div>

      <!-- Collect the nav links, forms, and other content for toggling -->
    <div class="collapse navbar-collapse" id="navbar-collapse">
      <ul class="nav navbar-nav">
        {% include "header/_context_selection.html" %}
      </ul>

      <ul class="nav navbar-nav navbar-right">
        <li id="extensible-header" class="extensible-header"></li>
        {% if profiler_enabled %}
          {% include "developer/profiler/_mode_picker.html" %}
        {% endif %}
        {% include "header/_user_menu.html" %}
        {% include "header/_region_selection.html" %}
      </ul>
    </div><!-- /.navbar-collapse -->
  </div><!-- /.container-fluid -->
</nav>
