{% extends "horizon/client_side/template.html" %}
{% load horizon %}

{% block id %}modal_template{% endblock %}

{% block template %}{% spaceless %}{% jstemplate %}
<div class="modal" data-backdrop="[[modal_backdrop]]">
  <div class="modal-dialog">
    <div class="modal-content">
      <div class='modal-header'>
        <a class='close' data-dismiss='modal' href="#">
          <span class="fa fa-times"></span>
        </a>
        <h3 class="modal-title">[[title]]</h3>
      </div>
      <div class='modal-body'>
        [[[body]]]
      </div>
      <div class='modal-footer'>
        <a href='#' class='btn btn-default cancel' data-dismiss='modal'>[[cancel]]</a>
        <a href='#' class='btn [[confirmCssClass]]'>[[confirm]]</a>
      </div>
    </div>
  </div>
</div>
{% endjstemplate %}{% endspaceless %}{% endblock %}
